@@ -858,7 +858,7 @@ GEIS_API GeisStatus geis_set_configuration(Geis        geis,
 
 typedef unsigned int GeisInputDeviceId;
 
-#define GEIS_ALL_INPUT_DEVICES ((GeisInputDeviceId)0)
+#define GEIS_ALL_INPUT_DEVICES ((GeisInputDeviceId*)0)
 
 /**
  * Prototype for input device callback functions.
