@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright 2011, 2012 Canonical Ltd.
+ * Copyright 2011-2013 Canonical Ltd.
  *
  * This library is free software: you can redistribute it and/or modify it 
  * under the terms of the GNU Lesser General Public License version 3
@@ -423,7 +423,7 @@ _grail_be_slice_state_update(struct _GeisSliceState *slice_state, UGSlice slice)
     slice_state->angle = 0.0f;
     slice_state->position_x = grail_slice_get_original_center_x(slice) +
                               (*C)[0][2];
-    slice_state->position_x = grail_slice_get_original_center_x(slice) +
+    slice_state->position_y = grail_slice_get_original_center_y(slice) +
                               (*C)[1][2];
     slice_state->radius = 1.0f;
     slice_state->num_touches = grail_slice_get_num_touches(slice);
@@ -674,8 +674,10 @@ _grail_be_grail_fd_callback(int                             fd GEIS_UNUSED,
         focus_y = (*C)[1][2] + grail_slice_get_original_center_y(slice);
       }
 
-      GeisFloat bboxMinX, bboxMinY = FLT_MAX;
-      GeisFloat bboxMaxX, bboxMaxY = 0.0f;
+      GeisFloat bboxMinX = FLT_MAX;
+      GeisFloat bboxMinY = FLT_MAX;
+      GeisFloat bboxMaxX = 0.0f;
+      GeisFloat bboxMaxY = 0.0f;
       for (GeisInteger t = 0; t < num_touches; ++t)
       {
         GeisTouch touch = geis_touch_new(t);
@@ -707,7 +709,7 @@ _grail_be_grail_fd_callback(int                             fd GEIS_UNUSED,
           focus_y = frame_touch_get_window_y(uftouch);
         }
 
-        GeisFloat touch_x, touch_y;
+        GeisFloat touch_x = 0.0f, touch_y = 0.0f;
         if (is_touchscreen)
         {
           touch_x = frame_touch_get_window_x(uftouch);
@@ -810,11 +812,14 @@ _grail_be_subscribe_new_device(GeisGrailBackend gbe, GeisDevice device)
       GeisBoolean device_applies = GEIS_TRUE;
       for (GeisSize tindex = 0; tindex < geis_filter_term_count(*fit); ++tindex)
       {
-        if (!geis_filter_term_match_device(geis_filter_term(*fit, tindex),
-                                           device))
+        GeisFilterTerm term = geis_filter_term(*fit, tindex);
+        if (geis_filter_term_facility(term) == GEIS_FILTER_DEVICE)
         {
-          device_applies = GEIS_FALSE;
-          break;
+          if (!geis_filter_term_match_device(term, device))
+          {
+            device_applies = GEIS_FALSE;
+            break;
+          }
         }
       }
 
@@ -1011,7 +1016,6 @@ _geis_grail_add_device(GeisGrailBackend gbe, UFDevice frame_device)
                              / sizeof(struct GeisFilterableAttribute);
 
   geis_register_device(gbe->geis, geis_device, attr_count, attrs);
-  _grail_be_subscribe_new_device(gbe, geis_device);
 
   /* We are not going to hold a pointer to this geis_device ourselves  */
   geis_device_unref(geis_device);
@@ -1810,6 +1814,13 @@ _grail_be_get_configuration(GeisBackend      be GEIS_UNUSED,
     return retval;
   }
 
+  if (0 == strcmp(item_name, GEIS_CONFIG_NUM_ACTIVE_SUBSCRIPTIONS))
+  {
+    struct GeisGrailSubscriptionData *sub_data = geis_subscription_pdata(subscription);
+    *((GeisSize*)item_value) = geis_ugsubscription_count(sub_data->ugstore);
+    retval =  GEIS_STATUS_SUCCESS;
+  }
+
   #define GEIS_GRAIL_CHECK_GESTURE_CONFIG(gesture, Gesture, GESTURE) \
   if (strcmp(item_name, GEIS_CONFIG_##GESTURE##_TIMEOUT) == 0) \
   { \
@@ -1828,7 +1839,7 @@ _grail_be_get_configuration(GeisBackend      be GEIS_UNUSED,
         item_value); \
   }
 
-  GEIS_GRAIL_CHECK_GESTURE_CONFIG(drag, Drag, DRAG)
+  else GEIS_GRAIL_CHECK_GESTURE_CONFIG(drag, Drag, DRAG)
   else GEIS_GRAIL_CHECK_GESTURE_CONFIG(pinch, Pinch, PINCH)
   else GEIS_GRAIL_CHECK_GESTURE_CONFIG(rotate, Rotate, ROTATE)
   else GEIS_GRAIL_CHECK_GESTURE_CONFIG(tap, Tap, TAP)
@@ -1919,12 +1930,36 @@ _grail_be_set_configuration(GeisBackend      be GEIS_UNUSED,
   return retval;
 }
 
+static GeisStatus
+_grail_be_activate_device(GeisBackend be, GeisDevice device)
+{
+  GeisStatus status = GEIS_STATUS_SUCCESS;
+  GeisGrailBackend gbe = (GeisGrailBackend)be;
+  _grail_be_subscribe_new_device(gbe, device);
+  return status;
+}
+
+
+static GeisStatus
+_grail_be_deactivate_device(GeisBackend be, GeisDevice device)
+{
+  GeisStatus status = GEIS_STATUS_SUCCESS;
+  GeisGrailBackend gbe = (GeisGrailBackend)be;
+  UFDevice ufdevice = _grail_be_ufdevice_from_device_id(gbe,
+                                                geis_device_id(device));
+  _grail_be_unsubscribe_removed_device(gbe, ufdevice);
+  return status;
+}
+
+
 static struct GeisBackendVtable gbe_vtbl = {
   _geis_grail_backend_construct,
   _geis_grail_backend_finalize,
   geis_grail_token_new,
   _grail_be_accept_gesture,
   _grail_be_reject_gesture,
+  _grail_be_activate_device,
+  _grail_be_deactivate_device,
   _grail_be_get_configuration,
   _grail_be_set_configuration
 };
