@@ -2,7 +2,7 @@
  * @file geis_backend.c
  * @brief internal GEIS back end base class implementation
  *
- * Copyright 2010 Canonical Ltd.
+ * Copyright 2010-2013 Canonical Ltd.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of version 3 of the GNU Lesser General Public License as published
@@ -199,6 +199,26 @@ geis_backend_gesture_reject(GeisBackend   be,
 }
 
 
+GeisStatus
+geis_backend_activate_device(GeisBackend  be,
+                             GeisDevice   device)
+{
+  if (be->be_class->vtbl->activate_device)
+    return  be->be_class->vtbl->activate_device(_data_from_be(be), device);
+  return GEIS_STATUS_SUCCESS;
+}
+
+
+GeisStatus
+geis_backend_deactivate_device(GeisBackend  be,
+                               GeisDevice   device)
+{
+  if (be->be_class->vtbl->deactivate_device)
+    return  be->be_class->vtbl->deactivate_device(_data_from_be(be), device);
+  return GEIS_STATUS_SUCCESS;
+}
+
+
 /*
  * Gets a back end configuration value.
  */
