@@ -2,7 +2,7 @@
  * @file geis_backend.h
  * @brief internal GEIS back end base class public interface
  *
- * Copyright 2010, 2012 Canonical Ltd.
+ * Copyright 2010-2013 Canonical Ltd.
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of version 3 of the GNU Lesser General Public License as published
@@ -89,6 +89,34 @@ geis_backend_gesture_reject(GeisBackend   be,
                             GeisGroup     group,
                             GeisGestureId gesture_id);
 
+/**
+ * Tells the back end to activate a device after it's been set up.
+ *
+ * @param[in] be          The GEIS back end.
+ * @param[in] device      The GEIS device for which subscription wll activate.
+ *
+ * The back end sends and event to the middle end when a new device has been
+ * detected.  The middle end sets up some structures etc., then tells the back
+ * end to activate the device (which may actually cause some subscriptions to be
+ * activated), then forwards the new-device event to the front end where the
+ * application can be made aware of it.
+ */
+GeisStatus
+geis_backend_activate_device(GeisBackend  be,
+                             GeisDevice   device);
+
+/**
+ * Tells the back end to activate a device after it's been set up.
+ *
+ * @param[in] be          The GEIS back end.
+ * @param[in] device      The GEIS device for which subscription wll deactivate.
+ *
+ * See geis_backend_activate_device for more information.
+ */
+GeisStatus
+geis_backend_deactivate_device(GeisBackend  be,
+                               GeisDevice   device);
+
 /**
  * Gets a back end configuration value.
  *
