@@ -5,9 +5,8 @@
  * Copyright 2010 Canonical Ltd.
  *
  * This library is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by the Free
- * Software Foundation; either version 3 of the License, or (at your option) any
- * later version.
+ * the terms of version 3 of the GNU Lesser General Public License as published
+ * by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -30,6 +29,14 @@
  */
 GeisEvent geis_event_new(GeisEventType type);
 
+/**
+ * Overrides the type of the event.
+ *
+ * @param[in] event The event.
+ * @param[in] type  The type of the event.
+ */
+void geis_event_override_type(GeisEvent event, GeisEventType type);
+
 /**
  * Adds an attr to an event.
  *
