@@ -5,9 +5,8 @@
  * Copyright 2011 Canonical Ltd.
  *
  * This library is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by the Free
- * Software Foundation; either version 3 of the License, or (at your option) any
- * later version.
+ * the terms of version 3 of the GNU Lesser General Public License as published
+ * by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -18,6 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
+#include <errno.h>
 #include "geis/geis.h"
 #include "geis_class.h"
 #include "geis_config.h"
@@ -25,6 +25,7 @@
 #include "geis_logging.h"
 #include "geis_private.h"
 #include "geis_subscription.h"
+#include "geis_test_api.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,52 +42,70 @@ struct _GeisInstance
   GeisPointer       input_context;
   GeisGestureFuncs  gesture_funcs;
   GeisPointer       gesture_cookie;
+  GeisBoolean       init_failed;
 };
 
 /* maps GEIS v1 gesture names into GEIS v2 gesture attrs */
 typedef struct _GeisV1AttrMap *GeisV1AttrMap;
-struct _GeisV1AttrMap
+static struct _GeisV1AttrMap
 {
-  const char *v1_type;
+  const char* v1_type;
   const char* v2_name;
+  GeisInteger v2_id;
   GeisInteger touches;
 } s_v1_attr_map[] =
 {
-  { GEIS_GESTURE_TYPE_DRAG1,   GEIS_GESTURE_DRAG,   1 },
-  { GEIS_GESTURE_TYPE_DRAG2,   GEIS_GESTURE_DRAG,   2 },
-  { GEIS_GESTURE_TYPE_DRAG3,   GEIS_GESTURE_DRAG,   3 },
-  { GEIS_GESTURE_TYPE_DRAG4,   GEIS_GESTURE_DRAG,   4 },
-  { GEIS_GESTURE_TYPE_DRAG5,   GEIS_GESTURE_DRAG,   5 },
-  { GEIS_GESTURE_TYPE_PINCH1,  GEIS_GESTURE_PINCH,  1 },
-  { GEIS_GESTURE_TYPE_PINCH2,  GEIS_GESTURE_PINCH,  2 },
-  { GEIS_GESTURE_TYPE_PINCH3,  GEIS_GESTURE_PINCH,  3 },
-  { GEIS_GESTURE_TYPE_PINCH4,  GEIS_GESTURE_PINCH,  4 },
-  { GEIS_GESTURE_TYPE_PINCH5,  GEIS_GESTURE_PINCH,  5 },
-  { GEIS_GESTURE_TYPE_ROTATE1, GEIS_GESTURE_ROTATE, 1 },
-  { GEIS_GESTURE_TYPE_ROTATE2, GEIS_GESTURE_ROTATE, 2 },
-  { GEIS_GESTURE_TYPE_ROTATE3, GEIS_GESTURE_ROTATE, 3 },
-  { GEIS_GESTURE_TYPE_ROTATE4, GEIS_GESTURE_ROTATE, 4 },
-  { GEIS_GESTURE_TYPE_ROTATE5, GEIS_GESTURE_ROTATE, 5 },
-  { GEIS_GESTURE_TYPE_TAP1,    GEIS_GESTURE_TAP,    1 },
-  { GEIS_GESTURE_TYPE_TAP2,    GEIS_GESTURE_TAP,    2 },
-  { GEIS_GESTURE_TYPE_TAP3,    GEIS_GESTURE_TAP,    3 },
-  { GEIS_GESTURE_TYPE_TAP4,    GEIS_GESTURE_TAP,    4 },
-  { GEIS_GESTURE_TYPE_TAP5,    GEIS_GESTURE_TAP,    5 },
-  { GEIS_GESTURE_TYPE_TOUCH1,  GEIS_GESTURE_TOUCH,  1 },
-  { GEIS_GESTURE_TYPE_TOUCH2,  GEIS_GESTURE_TOUCH,  2 },
-  { GEIS_GESTURE_TYPE_TOUCH3,  GEIS_GESTURE_TOUCH,  3 },
-  { GEIS_GESTURE_TYPE_TOUCH4,  GEIS_GESTURE_TOUCH,  4 },
-  { GEIS_GESTURE_TYPE_TOUCH5,  GEIS_GESTURE_TOUCH,  5 },
-  { GEIS_GESTURE_TYPE_FLICK1,  GEIS_GESTURE_FLICK,  1 },
-  { GEIS_GESTURE_TYPE_FLICK2,  GEIS_GESTURE_FLICK,  2 },
-  { GEIS_GESTURE_TYPE_FLICK3,  GEIS_GESTURE_FLICK,  3 },
-  { GEIS_GESTURE_TYPE_FLICK4,  GEIS_GESTURE_FLICK,  4 },
-  { GEIS_GESTURE_TYPE_FLICK5,  GEIS_GESTURE_FLICK,  5 },
-  { NULL,                      NULL,                0 }
+  { GEIS_GESTURE_TYPE_DRAG1,   GEIS_GESTURE_DRAG,   GEIS_GESTURE_PRIMITIVE_DRAG,   1 },
+  { GEIS_GESTURE_TYPE_DRAG2,   GEIS_GESTURE_DRAG,   GEIS_GESTURE_PRIMITIVE_DRAG,   2 },
+  { GEIS_GESTURE_TYPE_DRAG3,   GEIS_GESTURE_DRAG,   GEIS_GESTURE_PRIMITIVE_DRAG,   3 },
+  { GEIS_GESTURE_TYPE_DRAG4,   GEIS_GESTURE_DRAG,   GEIS_GESTURE_PRIMITIVE_DRAG,   4 },
+  { GEIS_GESTURE_TYPE_DRAG5,   GEIS_GESTURE_DRAG,   GEIS_GESTURE_PRIMITIVE_DRAG,   5 },
+  { GEIS_GESTURE_TYPE_PINCH1,  GEIS_GESTURE_PINCH,  GEIS_GESTURE_PRIMITIVE_PINCH,  1 },
+  { GEIS_GESTURE_TYPE_PINCH2,  GEIS_GESTURE_PINCH,  GEIS_GESTURE_PRIMITIVE_PINCH,  2 },
+  { GEIS_GESTURE_TYPE_PINCH3,  GEIS_GESTURE_PINCH,  GEIS_GESTURE_PRIMITIVE_PINCH,  3 },
+  { GEIS_GESTURE_TYPE_PINCH4,  GEIS_GESTURE_PINCH,  GEIS_GESTURE_PRIMITIVE_PINCH,  4 },
+  { GEIS_GESTURE_TYPE_PINCH5,  GEIS_GESTURE_PINCH,  GEIS_GESTURE_PRIMITIVE_PINCH,  5 },
+  { GEIS_GESTURE_TYPE_ROTATE1, GEIS_GESTURE_ROTATE, GEIS_GESTURE_PRIMITIVE_ROTATE, 1 },
+  { GEIS_GESTURE_TYPE_ROTATE2, GEIS_GESTURE_ROTATE, GEIS_GESTURE_PRIMITIVE_ROTATE, 2 },
+  { GEIS_GESTURE_TYPE_ROTATE3, GEIS_GESTURE_ROTATE, GEIS_GESTURE_PRIMITIVE_ROTATE, 3 },
+  { GEIS_GESTURE_TYPE_ROTATE4, GEIS_GESTURE_ROTATE, GEIS_GESTURE_PRIMITIVE_ROTATE, 4 },
+  { GEIS_GESTURE_TYPE_ROTATE5, GEIS_GESTURE_ROTATE, GEIS_GESTURE_PRIMITIVE_ROTATE, 5 },
+  { GEIS_GESTURE_TYPE_TAP1,    GEIS_GESTURE_TAP,    GEIS_GESTURE_PRIMITIVE_TAP,    1 },
+  { GEIS_GESTURE_TYPE_TAP2,    GEIS_GESTURE_TAP,    GEIS_GESTURE_PRIMITIVE_TAP,    2 },
+  { GEIS_GESTURE_TYPE_TAP3,    GEIS_GESTURE_TAP,    GEIS_GESTURE_PRIMITIVE_TAP,    3 },
+  { GEIS_GESTURE_TYPE_TAP4,    GEIS_GESTURE_TAP,    GEIS_GESTURE_PRIMITIVE_TAP,    4 },
+  { GEIS_GESTURE_TYPE_TAP5,    GEIS_GESTURE_TAP,    GEIS_GESTURE_PRIMITIVE_TAP,    5 },
+  { GEIS_GESTURE_TYPE_TOUCH1,  GEIS_GESTURE_TOUCH,  GEIS_GESTURE_PRIMITIVE_TOUCH,  1 },
+  { GEIS_GESTURE_TYPE_TOUCH2,  GEIS_GESTURE_TOUCH,  GEIS_GESTURE_PRIMITIVE_TOUCH,  2 },
+  { GEIS_GESTURE_TYPE_TOUCH3,  GEIS_GESTURE_TOUCH,  GEIS_GESTURE_PRIMITIVE_TOUCH,  3 },
+  { GEIS_GESTURE_TYPE_TOUCH4,  GEIS_GESTURE_TOUCH,  GEIS_GESTURE_PRIMITIVE_TOUCH,  4 },
+  { GEIS_GESTURE_TYPE_TOUCH5,  GEIS_GESTURE_TOUCH,  GEIS_GESTURE_PRIMITIVE_TOUCH,  5 },
+  { GEIS_GESTURE_TYPE_FLICK1,  GEIS_GESTURE_FLICK,  100,                           1 },
+  { GEIS_GESTURE_TYPE_FLICK2,  GEIS_GESTURE_FLICK,  100,                           2 },
+  { GEIS_GESTURE_TYPE_FLICK3,  GEIS_GESTURE_FLICK,  100,                           3 },
+  { GEIS_GESTURE_TYPE_FLICK4,  GEIS_GESTURE_FLICK,  100,                           4 },
+  { GEIS_GESTURE_TYPE_FLICK5,  GEIS_GESTURE_FLICK,  100,                           5 },
+  { NULL,                      NULL,                0,                             0 }
 };
 
+static const GeisString UNKNOWN_GESTURE_TYPE = "(unknown)";
 
-struct { const char *id; const char* x; const char* y; } s_touch_attr_names[] =
+static GeisString
+_v1_map_v2_name_to_v1_name(GeisString v2_name, GeisInteger touches)
+{
+  for (int i = 0; s_v1_attr_map[i].v1_type; ++i)
+  {
+    if (0 == strcmp(s_v1_attr_map[i].v2_name, v2_name)
+     && s_v1_attr_map[i].touches == touches)
+    {
+      return s_v1_attr_map[i].v1_type;
+    }
+  }
+  return UNKNOWN_GESTURE_TYPE;
+}
+
+
+static struct { const char *id; const char* x; const char* y; } s_touch_attr_names[] =
 {
   {
     GEIS_GESTURE_ATTRIBUTE_TOUCH_0_ID,
@@ -147,7 +166,7 @@ _map_frame_to_attrs(Geis          geis,
 
   *attr_count = 0;
 
-  geis_debug("num_attrs = %lu", num_attrs);
+  geis_debug("num_attrs = %zu", num_attrs);
   GeisGestureAttr *attrs = calloc(gga_size, sizeof(GeisGestureAttr));
   if (!attrs)
   {
@@ -183,7 +202,7 @@ _map_frame_to_attrs(Geis          geis,
     GeisAttr attr = geis_frame_attr(frame, i);
     GeisString attr_name = geis_attr_name(attr);
 
-    if (0 == strcmp(attr_name, "geis gesture class id"))
+    if (0 == strcmp(attr_name, GEIS_CLASS_ATTRIBUTE_ID))
     {
       /* skip this, do not pass it on */
     }
@@ -240,8 +259,7 @@ _map_frame_to_attrs(Geis          geis,
     {
       attrs[7].name       = attr_name;
       attrs[7].type       = GEIS_ATTR_TYPE_STRING;
-      attrs[7].string_val = gesture_name ? gesture_name
-                                         : geis_attr_value_to_string(attr);
+      if (!gesture_name) gesture_name = geis_attr_value_to_string(attr);
       ++*attr_count;
     }
     else if (0 == strcmp(attr_name, GEIS_GESTURE_ATTRIBUTE_TOUCHES))
@@ -262,6 +280,9 @@ _map_frame_to_attrs(Geis          geis,
     }
   }
 
+  /* fix up the gesture name for backwards compatibility */
+  attrs[7].string_val = _v1_map_v2_name_to_v1_name(gesture_name, num_touches);
+
   for (i = 0; i < geis_frame_touchid_count(frame); ++i)
   {
     GeisTouch touch = geis_touchset_touch_by_id(touchset,
@@ -310,8 +331,32 @@ _v1_event_callback(Geis        geis,
                    GeisEvent   event,
                    GeisPointer context)
 {
-  GeisInteger class_id = 0;
   GeisInstance v1_instance = (GeisInstance)context;
+  switch (geis_event_type(event))
+  {
+    case GEIS_EVENT_INIT_COMPLETE:
+      geis_debug("received INIT event");
+      goto final_exit;
+
+    case GEIS_EVENT_ERROR:
+      geis_debug("received ERROR event");
+      v1_instance->init_failed = GEIS_TRUE;
+      goto final_exit;
+
+    case GEIS_EVENT_DEVICE_AVAILABLE:
+    case GEIS_EVENT_DEVICE_UNAVAILABLE:
+    case GEIS_EVENT_CLASS_AVAILABLE:
+    case GEIS_EVENT_CLASS_CHANGED:
+    case GEIS_EVENT_CLASS_UNAVAILABLE:
+      geis_debug("received DEVICE/CLASS event");
+      goto final_exit;
+
+    default:
+      geis_debug("received GESTURE event");
+      break;
+  }
+
+  GeisInteger class_id = 0;
   GeisAttr attr = geis_event_attr_by_name(event, GEIS_EVENT_ATTRIBUTE_TOUCHSET);
   if (!attr)
   {
@@ -346,7 +391,7 @@ _v1_event_callback(Geis        geis,
     GeisGroup group = geis_groupset_group(groupset, i);
     if (!group)
     {
-      geis_warning("can not extract group %lu from groupset", i);
+      geis_warning("can not extract group %zu from groupset", i);
       goto final_exit;
     }
 
@@ -357,7 +402,7 @@ _v1_event_callback(Geis        geis,
       GeisFrame frame = geis_group_frame(group, j);
       if (!frame)
       {
-	geis_warning("can not extract frame %lu from group", j);
+	geis_warning("can not extract frame %zu from group", j);
 	goto final_exit;
       }
 
@@ -435,23 +480,24 @@ geis_init(GeisWinInfo* win_info, GeisInstance *geis_instance)
   if (win_info->win_type == geis_win_type_str(Test))
   {
     if (xcb_win_info)
-      instance->geis = geis_new(GEIS_INIT_OIF_MOCK_ENGINE,
+      instance->geis = geis_new(GEIS_INIT_MOCK_BACKEND,
                                 GEIS_INIT_TRACK_GESTURE_CLASSES,
                                 NULL);
     else
-      instance->geis = geis_new(GEIS_INIT_OIF_MOCK_ENGINE, NULL);
+      instance->geis = geis_new(GEIS_INIT_MOCK_BACKEND, NULL);
   }
   else
   {
-    instance->geis = geis_new(GEIS_INIT_OIF_XCB,
-                              GEIS_INIT_TRACK_DEVICES,
+    instance->geis = geis_new(GEIS_INIT_TRACK_DEVICES,
                               GEIS_INIT_TRACK_GESTURE_CLASSES,
+                              GEIS_INIT_SYNCHRONOUS_START,
                               NULL);
   }
   if (!instance->geis)
   {
     goto unwind_instance;
   }
+
   geis_register_event_callback(instance->geis, _v1_event_callback, instance);
 
   instance->subscription = geis_subscription_new(instance->geis,
@@ -465,6 +511,8 @@ geis_init(GeisWinInfo* win_info, GeisInstance *geis_instance)
   status = geis_subscription_add_filter(instance->subscription,
                                         instance->window_filter);
 
+  instance->init_failed = GEIS_FALSE;
+
   *geis_instance = instance;
   goto final_exit;
 
@@ -480,7 +528,6 @@ geis_finish(GeisInstance instance)
 {
   GeisStatus status = GEIS_STATUS_SUCCESS;
   geis_subscription_delete(instance->subscription);
-  geis_filter_delete(instance->window_filter);
   geis_delete(instance->geis);
   free(instance);
   return status;
@@ -558,9 +605,9 @@ geis_event_dispatch(GeisInstance instance)
 
 
 static GeisStatus
-_subscribe_device(GeisInstance instance,
-                  GeisInteger device_id,
-                  const char **gesture_list)
+_v1_subscribe_device(GeisInstance instance,
+                     GeisInteger device_id,
+                     const char **gesture_list)
 {
   GeisStatus result = GEIS_UNKNOWN_ERROR;
   if (gesture_list == GEIS_ALL_GESTURES)
@@ -572,15 +619,6 @@ _subscribe_device(GeisInstance instance,
     const char **g;
 
     geis_debug("subscribing device %d for the following gestures:", device_id);
-    result = geis_filter_add_term(instance->window_filter,
-                        GEIS_FILTER_DEVICE,
-                        GEIS_DEVICE_ATTRIBUTE_ID, GEIS_FILTER_OP_EQ, device_id,
-                        NULL);
-    if (result != GEIS_STATUS_SUCCESS)
-    {
-      geis_error("error adding device filter term");
-    }
-
     for (g = gesture_list; *g; ++g)
     {
       GeisV1AttrMap v1attr;
@@ -605,7 +643,7 @@ _subscribe_device(GeisInstance instance,
 	  else
 	  {
 	    result = geis_filter_add_term(filter, GEIS_FILTER_CLASS,
-	      GEIS_CLASS_ATTRIBUTE_NAME, GEIS_FILTER_OP_EQ, v1attr->v2_name,
+	      GEIS_CLASS_ATTRIBUTE_ID, GEIS_FILTER_OP_EQ, v1attr->v2_id,
 	      GEIS_GESTURE_ATTRIBUTE_TOUCHES, GEIS_FILTER_OP_EQ, v1attr->touches,
 	      NULL);
 	    if (result != GEIS_STATUS_SUCCESS)
@@ -613,25 +651,27 @@ _subscribe_device(GeisInstance instance,
 	      geis_error("error adding gesture class filter term");
 	    }
 
+            if (device_id != 0)
+            {
+              result = geis_filter_add_term(filter,
+                                  GEIS_FILTER_DEVICE, GEIS_DEVICE_ATTRIBUTE_ID, GEIS_FILTER_OP_EQ,
+                                  device_id, NULL);
+              if (result != GEIS_STATUS_SUCCESS)
+              {
+                geis_error("error adding device filter term");
+              }
+            }
+
 	    result = geis_subscription_add_filter(instance->subscription, filter); 
 	    if (result != GEIS_STATUS_SUCCESS)
 	    {
 	      geis_error("error adding gesture class filter");
 	    }
-	    geis_filter_delete(filter);
 	  }
 	  break;
 	}
       }
     }
-
-    /* remove the window-id-only filter since it allows ALL gesture classes */
-    result = geis_subscription_remove_filter(instance->subscription, 
-                                             instance->window_filter);
-    if (result != GEIS_STATUS_SUCCESS)
-    {
-      geis_warning("error removing V1 window filter");
-    }
   }
   return result;
 }
@@ -646,6 +686,16 @@ geis_subscribe(GeisInstance         instance,
 {
   GeisStatus result = GEIS_UNKNOWN_ERROR;
 
+  /**
+   * If there is no window filter, the instance has already been subscribed to
+   * devices and gestures so skip.
+   */
+  if (NULL == instance->window_filter)
+  {
+    geis_warning("instance has been subscribed twice");
+    return GEIS_STATUS_SUCCESS;
+  }
+
   memcpy(&instance->gesture_funcs, funcs, sizeof(GeisGestureFuncs));
   instance->gesture_cookie = cookie;
 
@@ -655,20 +705,37 @@ geis_subscribe(GeisInstance         instance,
 
   if (input_list == GEIS_ALL_INPUT_DEVICES)
   {
-    result = _subscribe_device(instance, 0, gesture_list);
+    result = _v1_subscribe_device(instance, 0, gesture_list);
   }
   else
   {
     GeisInputDeviceId *device_id;
     for (device_id = input_list; *device_id; ++device_id)
     {
-      GeisStatus a_result = _subscribe_device(instance, *device_id, gesture_list);
+      GeisStatus a_result = _v1_subscribe_device(instance, *device_id, gesture_list);
       if (a_result == GEIS_STATUS_SUCCESS)
       {
-	result = a_result;
+        result = a_result;
       }
     }
   }
+
+  /*
+   * If there are specific gesture classes, remove the window-id-only filter since it allows
+   * ALL gesture classes.  It will have been replaced by device-specific or
+   * class-specific filters on the same window.
+   */
+  if (gesture_list && *gesture_list)
+  {
+    result = geis_subscription_remove_filter(instance->subscription, 
+                                             instance->window_filter);
+    if (result != GEIS_STATUS_SUCCESS)
+    {
+      geis_warning("error removing V1 window filter");
+    }
+    instance->window_filter = NULL;
+  }
+
   result = geis_subscription_activate(instance->subscription);
   return result;
 }
@@ -721,42 +788,27 @@ geis_unsubscribe(GeisInstance     instance,
 }
 
 
+/*
+ * Translates a GEISv2 device into a GEISv1 device and invokes the appropriate
+ * callback.
+ */
 static void
-_v1_input_callback(Geis       geis GEIS_UNUSED,
-                   GeisEvent  event,
-                   void      *context)
+_v1_report_device(GeisInstance instance,
+                  GeisDevice   device,
+                  GeisBoolean  is_added)
 {
-  GeisInstance v1_instance = (GeisInstance)context;
-  GeisDevice device;
-  GeisAttr attr;
-  GeisSize i;
   GeisInputDeviceId device_id = 0;
-
-  attr = geis_event_attr_by_name(event, GEIS_EVENT_ATTRIBUTE_DEVICE);
-  if (!attr)
-  {
-    geis_error("no touchset for gesture event");
-    goto final_exit;
-  }
-
-  device = geis_attr_value_to_pointer(attr);
-  if (!device)
-  {
-    geis_warning("can not convert attr to device");
-    goto final_exit;
-  }
-
   GeisGestureAttr *attrs = calloc(geis_device_attr_count(device) + 1,
                                   sizeof(GeisGestureAttr));
   if (!attrs)
   {
     geis_error("can not allocate device attrs");
-    goto final_exit;
+    return;
   }
 
-  for (i = 0; i < geis_device_attr_count(device); ++i)
+  for (GeisSize i = 0; i < geis_device_attr_count(device); ++i)
   {
-    attr = geis_device_attr(device, i);
+    GeisAttr attr = geis_device_attr(device, i);
     if (0 == strcmp(geis_attr_name(attr), GEIS_DEVICE_ATTRIBUTE_ID))
     {
       device_id = geis_attr_value_to_integer(attr);
@@ -782,25 +834,59 @@ _v1_input_callback(Geis       geis GEIS_UNUSED,
     }
   }
 
+  if (is_added)
+  {
+    instance->input_funcs.added(instance->input_context, device_id, attrs);
+  }
+  else
+  {
+    instance->input_funcs.removed(instance->input_context, device_id, attrs);
+  }
+
+  free(attrs);
+}
+
+
+/*
+ * Handler for device events.
+ *
+ * Converts GEISv2 device-add and device-removed events into similar GEISv1
+ * callbacks.
+ */
+static void
+_v1_input_callback(Geis       geis GEIS_UNUSED,
+                   GeisEvent  event,
+                   void      *context)
+{
+  GeisInstance instance = (GeisInstance)context;
+
+  GeisAttr attr = geis_event_attr_by_name(event, GEIS_EVENT_ATTRIBUTE_DEVICE);
+  if (!attr)
+  {
+    geis_error("no touchset for gesture event");
+    goto final_exit;
+  }
+
+  GeisDevice device = geis_attr_value_to_pointer(attr);
+  if (!device)
+  {
+    geis_warning("can not convert attr to device");
+    goto final_exit;
+  }
+
   switch (geis_event_type(event))
   {
     case GEIS_EVENT_DEVICE_AVAILABLE:
-      v1_instance->input_funcs.added(v1_instance->input_context,
-                                      device_id,
-                                      attrs);
+      _v1_report_device(instance, device, GEIS_TRUE);
       break;
     case GEIS_EVENT_DEVICE_UNAVAILABLE:
-      v1_instance->input_funcs.removed(v1_instance->input_context,
-                                        device_id,
-                                        attrs);
+      _v1_report_device(instance, device, GEIS_FALSE);
       break;
     default:
       geis_debug("-- event ignored --");
       break;
   }
 
-  free(attrs);
-
 final_exit:
   geis_event_delete(event);
 }
@@ -818,6 +904,14 @@ geis_input_devices(GeisInstance    instance,
 
   geis_register_device_callback(instance->geis, _v1_input_callback, instance);
 
+  /* report pre-existing devices */
+  GeisDeviceBag device_bag = geis_devices(instance->geis);
+  for (GeisSize d = 0; d != geis_device_bag_count(device_bag); ++d)
+  {
+    GeisDevice device = geis_device_bag_device(device_bag, d);
+    _v1_report_device(instance, device, GEIS_TRUE);
+  }
+
   return result;
 }
 
