@@ -1,9 +1,8 @@
 # Copyright (C) 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of version 3 of the GNU General Public License as
+# published by the Free Software Foundation.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +13,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+
+
 """Python bindings for the GEIS v2 gesture recognition interface.
 
 Do not use this module directly -- the public API is provided by the 'geis'
@@ -121,11 +122,12 @@ try:
     _geis_frame_attr_count = _geis_lib.geis_frame_attr_count
     _geis_frame_attr = _geis_lib.geis_frame_attr
     _geis_frame_attr.restype = ctypes.c_void_p
+    _geis_frame_is_class = _geis_lib.geis_frame_is_class
     _geis_frame_touchid_count = _geis_lib.geis_frame_touchid_count
     _geis_frame_touchid = _geis_lib.geis_frame_touchid
 
 except AttributeError as ex:
-    print ex
+    print(ex)
     sys.exit(1)
 
 
@@ -136,15 +138,19 @@ class GestureClass(object):
     def __init__(self, attr):
         self._class = _geis_attr_value_to_pointer(attr)
         self._id = _geis_gesture_class_id(self._class)
-        self._name = _geis_gesture_class_name(self._class)
+        self._name = _geis_gesture_class_name(self._class).decode('utf-8')
         self._attr_count = _geis_gesture_class_attr_count(self._class)
         self._attrs = {}
         for i in range(self._attr_count):
             attr = _geis_gesture_class_attr(self._class, i)
-            attr_name = _geis_attr_name(attr)
+            attr_name = _geis_attr_name(attr).decode('utf-8')
             self._attrs[attr_name] = _attr_types.get(_geis_attr_type(attr),
                                        _attr_type_unknown)(attr)
 
+    @property
+    def _as_parameter_(self):
+        return self._class
+
     def id(self):
         """ Gets the gesture class's system identifier (an integer).
         """
@@ -168,7 +174,7 @@ class Device(object):
     def __init__(self, attr):
         self._device = _geis_attr_value_to_pointer(attr)
         self._id = _geis_device_id(self._device)
-        self._name = _geis_device_name(self._device)
+        self._name = _geis_device_name(self._device).decode('utf-8')
         self._min_x = 0.0
         self._min_y = 0.0
         self._max_x = 0.0
@@ -177,7 +183,7 @@ class Device(object):
         self._attrs = {}
         for i in range(self._attr_count):
             attr = _geis_device_attr(self._device, i)
-            attr_name = _geis_attr_name(attr)
+            attr_name = _geis_attr_name(attr).decode('utf-8')
             self._attrs[attr_name] = _attr_types.get(_geis_attr_type(attr),
                                        _attr_type_unknown)(attr)
             if re.search("Abs MT Position X \d+ min", attr_name):
@@ -230,7 +236,7 @@ class Touch(object):
         attr_list = {}
         for i in range(self._attr_count):
             attr = _geis_touch_attr(self._touch, i)
-            attr_name = _geis_attr_name(attr)
+            attr_name = _geis_attr_name(attr).decode('utf-8')
             attr_list[attr_name] = _attr_types.get(_geis_attr_type(attr),
                                        _attr_type_unknown)(attr)
         return attr_list
@@ -250,7 +256,7 @@ class Touchset(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         """ Gets the next touch in the touchset as an iterable.
         """
         if self._index == self._count:
@@ -274,7 +280,7 @@ class Groupset(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         """ Gets the next group in the groupset as an iterable.
         """
         if self._index == self._count:
@@ -301,7 +307,7 @@ class Group(object):
     def id(self):
         return _geis_lib.geis_group_id(self._group)
 
-    def next(self):
+    def __next__(self):
         """ Gets the next frame in the group as an iterable.
         """
         if self._index == self._count:
@@ -324,11 +330,14 @@ class Frame(object):
         attr_list = {}
         for i in range(_geis_frame_attr_count(self._frame)):
             attr = _geis_frame_attr(self._frame, i)
-            attr_name = _geis_attr_name(attr)
+            attr_name = _geis_attr_name(attr).decode('utf-8')
             attr_list[attr_name] = _attr_types.get(_geis_attr_type(attr),
                                           _attr_type_unknown)(attr)
         return attr_list
 
+    def is_class(self, gclass):
+        return _geis_frame_is_class(self._frame, gclass)
+
     def touches(self):
         touch_list = []
         for i in range(_geis_frame_touchid_count(self._frame)):
@@ -361,7 +370,7 @@ def _pointer_type_touchset(attr):
 
 
 def _pointer_type_unknown(attr):
-    print "unknown pointer type", _geis_attr_name(attr)
+    print("unknown pointer type ")
     return None
 
 
@@ -396,7 +405,8 @@ def _attr_type_integer(attr):
 def _attr_type_pointer(attr):
     """ Extracts an attribute value as an object.
     """
-    return _attr_pointer_type.get(_geis_attr_name(attr), _pointer_type_unknown)(attr)
+    return _attr_pointer_type.get(_geis_attr_name(attr).decode('utf-8'),
+                                  _pointer_type_unknown)(attr)
 
 
 def _attr_type_string(attr):
@@ -408,7 +418,7 @@ def _attr_type_string(attr):
 
 
 def _attr_type_unknown(attr):
-    print "unknown attr type", _geis_attr_name(attr)
+    print("unknown attr type" + _geis_attr_name(attr).decode('utf-8'))
     return None
 
 
@@ -451,6 +461,8 @@ class Geis(object):
         terminator = 0,
         va_args = args + terminator
         self._instance = _geis_new(*va_args)
+        if self._instance == 0:
+            raise GeisError('can not create GEIS instance')
         self._cb_wrapper =  _EventCallback(_cb_wrapper)
         self._class_curry = None
         self._device_curry = None
@@ -468,7 +480,7 @@ class Geis(object):
         if (item_name == _geis_bindings.GEIS_CONFIGURATION_FD):
             fd = ctypes.c_long()
             status = _geis_get_configuration(self._instance,
-                                             item_name,
+                                             item_name.encode('utf-8'),
                                              ctypes.byref(fd))
             if status != _geis_bindings.GEIS_STATUS_SUCCESS:
                 raise GeisError('error retrieving GEIS fd')
@@ -545,7 +557,7 @@ class Event(object):
         attr_list = {}
         for i in range(_geis_event_attr_count(self._event)):
             attr = _geis_event_attr(self._event, i)
-            attr_name = _geis_attr_name(attr)
+            attr_name = _geis_attr_name(attr).decode('utf-8')
             attr_list[attr_name] = _attr_types.get(_geis_attr_type(attr),
                     _attr_type_unknown)(attr)
         return attr_list
@@ -556,7 +568,7 @@ class Filter(object):
     """
 
     def __init__(self, geis, name):
-        self._filter  = _geis_lib.geis_filter_new(geis, name)
+        self._filter  = _geis_lib.geis_filter_new(geis, name.encode('utf-8'))
 
     def __del__(self):
         _geis_lib.geis_filter_delete(self._filter)
@@ -581,8 +593,7 @@ class Filter(object):
                 raise ValueError('invalid filter term')
             name, op, value = term
             _geis_lib.geis_filter_add_term(self._filter, facility,
-                                           name, op, value,
-                                           0)
+                                           name.encode('utf-8'), op, value, 0)
 
 
 class Subscription(object):
@@ -603,7 +614,8 @@ class Subscription(object):
         :raise: Exception
         """
         object.__init__(self)
-        self._sub = _geis_subscription_new(geis, "py", 0)
+        self._sub = _geis_subscription_new(geis, "py".encode('utf-8'), 0)
+        self._filters = []
 
     def __del__(self):
         _geis_subscription_delete(self._sub)
@@ -628,6 +640,9 @@ class Subscription(object):
 
         :param filter: the filter to add
         """
+        # keep a reference to the filter for Python doesn't garbage collect it
+        if filt not in self._filters:
+            self._filters.append(filt)
         _geis_lib.geis_subscription_add_filter(self._sub, filt)
 
     def remove_filter(self, filt):
