@@ -4,6 +4,7 @@
 #include <check.h>
 
 #include <geis/geis.h>
+#include "libgeis/geis_test_api.h"
 #include <string.h>
 
 
@@ -14,7 +15,7 @@ static Geis g_geis;
 static void
 construct_geis()
 {
-  g_geis = geis_new(GEIS_INIT_OIF_MOCK_ENGINE, NULL);
+  g_geis = geis_new(GEIS_INIT_MOCK_BACKEND, NULL);
 }
 
 /* fixture teardown */
