@@ -4,8 +4,9 @@
 #include <check.h>
 
 #include "geis/geis.h"
-#include "libgeis/geis_backend_token.h"
-#include "libgeis/geis_private.h"
+#include "geis_backend_token.h"
+#include "geis_private.h"
+#include "geis_test_api.h"
 
 
 /* fixtures */
@@ -15,7 +16,7 @@ static Geis g_geis;
 static void
 construct_geis()
 {
-  g_geis = geis_new(GEIS_INIT_OIF_MOCK_ENGINE, NULL);
+  g_geis = geis_new(GEIS_INIT_MOCK_BACKEND, NULL);
 }
 
 /* fixture teardown */
@@ -35,6 +36,7 @@ START_TEST(creation)
   fail_if(token1 == NULL, "failed to create token1");
   token2 = geis_backend_token_clone(token1);
   fail_if(token2 == NULL, "failed to create token2");
+  fail_if(token2 == token1, "token clone failed");
 
   geis_backend_token_compose(token1, token2);
 
